/* Copyright (C) 2011 con terra GmbH (http://www.conterra.de)
 * All rights reserved
 */
package org.n52.security.common.crypto;

import static org.n52.security.common.collections.CollectionUtil.contains;

import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Arrays;

import javax.net.ssl.X509TrustManager;

import org.n52.security.common.collections.AbstractFilter;

/**
 *
 */
public class KeyPairX509TrustManager implements X509TrustManager {

    private KeyPair m_keypair;

    public KeyPairX509TrustManager(KeyPair keypair) {
        if (keypair == null) {
            throw new IllegalArgumentException("<keypair> must not be null!");
        }
        m_keypair = keypair;
    }

    @Override
    public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        isTrusted(chain);
    }

    @Override
    public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        isTrusted(chain);
    }

    private void isTrusted(X509Certificate[] chain) throws CertificateException {
        final Certificate trustedCert = m_keypair.getCertificate();
        if (!contains(Arrays.asList(chain), new AbstractFilter<X509Certificate>() {
            @Override
            public boolean accept(X509Certificate instance) {
                return instance.equals(trustedCert);
            }
        })) {
            throw new CertificateException("certificate not trusted!");
        }
    }

    @Override
    public X509Certificate[] getAcceptedIssuers() {
        return new X509Certificate[] { (X509Certificate) m_keypair.getCertificate() };
    }

}
