/**
 * *****************************************************************************
 * Copyright (C) 2011 by 52 North Initiative for Geospatial Open Source Software
 * GmbH
 *
 * Contact: Andreas Wytzisk 52 North Initiative for Geospatial Open Source
 * Software GmbH Martin-Luther-King-Weg 24 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 *****************************************************************************
 */
package org.n52.security.common.crypto;

import java.security.SecureRandom;

public class SaltUtil {

    private static final SecureRandom RANDOM_SALT = new SecureRandom();

    /**
     * Creates a salt with length 16;
     *
     */
    public static byte[] createSalt() {
        return createSalt(16);
    }

    /**
     * Creates a salt with a given length.
     *
     */
    public static byte[] createSalt(int length) {
        byte sid[] = new byte[length];
        RANDOM_SALT.nextBytes(sid);
        return sid;
    }

    /**
     * Creates a random length salt (length is between 16 and 48).
     *
     */
    public static byte[] createSaltWithRandomLength() {
        return createSaltWithRandomLength(16, 48);
    }

    /**
     * Creates a random length salt, with a length between min and max.
     */
    public static byte[] createSaltWithRandomLength(int min, int max) {
        if (max < min) {
            throw new IllegalArgumentException("max must greater min!");
        }
        if (min == max) {
            return createSalt(min);
        }
        return createSalt(min + RANDOM_SALT.nextInt(max - min));
    }
}
