/*******************************************************************************
 * Copyright (C) 2010 by 52 North Initiative for Geospatial Open Source Software
 * GmbH
 *
 * Contact: Andreas Wytzisk 52 North Initiative for Geospatial Open Source
 * Software GmbH Martin-Luther-King-Weg 24 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/
package org.n52.security.common.util;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

public class OrderedKeyValuePairCollection implements KeyValuePairCollection {

    private List<KeyValuePair> m_pairs = new ArrayList<KeyValuePair>();

    public Iterator<KeyValuePair> iterator() {
        return m_pairs.iterator();
    }

    public Collection<String> keys() {
        Collection<String> keys = new ArrayList<String>();
        for (KeyValuePair pair : m_pairs) {
           keys.add(pair.getKey());
        }
        return keys;
    }

    public Collection<String> values(KeyMatcher key) {
        Collection<String> values = new ArrayList<String>();
        for (KeyValuePair pair : getAll(key)) {
            values.add(pair.getValue());
        }
        return values;

    }

    public void add(KeyValuePair pair) {
        m_pairs.add(pair);
    }

    public void addAll(Collection<KeyValuePair> pairs) {
        if (pairs == null) {
            throw new IllegalArgumentException("'null' value not allowed.");
        }
        m_pairs.addAll(pairs);
    }

    public KeyValuePair getUnique(KeyMatcher keyMatcher) throws NonUniqueException {

        KeyValuePairCollection possiblyNonUniquePairs = getAll(keyMatcher);

        if (possiblyNonUniquePairs.size() == 0) {
            return null;
        }

        if (possiblyNonUniquePairs.size() > 1) {
            throw new NonUniqueException("Multiple matching KeyValuePairs found for key <" + keyMatcher + ">.");
        }

        return possiblyNonUniquePairs.iterator().next();
    }

    @Override
    public boolean containsKey(KeyMatcher key) {
        return getAll(key).size() > 0;
    }

    public KeyValuePairCollection getAll(KeyMatcher keyMatcher) {

        KeyValuePairCollection pairsPerKey = new OrderedKeyValuePairCollection();
        for (KeyValuePair kvp : m_pairs) {
            if (keyMatcher.matches(kvp.getKey())) {
                pairsPerKey.add(kvp);
            }
        }
        return pairsPerKey;
    }

    public void updateUnique(KeyMatcher keyMatcher, KeyValuePair pair) {
        KeyValuePair matchingPair = getUnique(keyMatcher);
        if (matchingPair != null) {
            m_pairs.remove(matchingPair);
        }
        add(pair);
    }

    public boolean contains(KeyValuePair pair) {
        return m_pairs.contains(pair);
    }


    @Override
    public int delete(KeyMatcher keyMatcher) {
        KeyValuePairCollection pairs = getAll(keyMatcher);
        int counter = 0;
        for (KeyValuePair pair : pairs) {
            m_pairs.remove(pair);
            counter++;
        }

        return counter;
    }

    public int size() {
        return m_pairs.size();
    }
}
